<?php
if (! class_exists('SLPUML_AdminUI')) {
	/**
	 * Admin interface methods.
	 *
	 * @package StoreLocatorPlus\UserManagedLocations\AdminUI
	 * @author De B.A.A.T. <slp-uml@de-baat.nl>
	 * @copyright 2014 Charleston Software Associates, LLC - De B.A.A.T.
	 *
	 */
	class SLPUML_AdminUI {

		//----------------------------------
		// Properties
		//----------------------------------

		/**
		 * The add-on pack from whence we came.
		 * 
		 * @var \SLPUserManagedLocations $addon
		 */
		private $addon;

		/**
		 * The UserManagedLocations add-on pack.
		 * 
		 * @var \SLPUserManagedLocations $parent
		 */
		private $parent;

		/**
		 * A shorthand for the Super Extendo Data object.
		 *
		 * @var \Extendo_Data
		 */
		public $SEData;

		/**
		 * The base SLPlus object.
		 *
		 * @var \SLPlus $slplus
		 */
		private $slplus;

		/**
		 * User Manager object.
		 *
		 * @var \SLPUML_AdminUI_UserManager $usermanager
		 */
		private $usermanager;

		/**
		 * The settings object.
		 *
		 * @var \wpCSL_settings__slplus $Settings
		*/
		//public $Settings;

		//-------------------------------------
		// Methods : Class Aministration
		//-------------------------------------

		/**
		 * Instantiate an object of this class.
		 *
		 * @param mixed[] $params Admin UI properties.
		 */
		function __construct($params = null) {

			// Set properties based on constructor params,
			// if the property named in the params array is well defined.
			//
			if ($params !== null) {
				foreach ($params as $property=>$value) {
					if (property_exists($this,$property)) { $this->$property = $value; }
				}
			}
		}


		//-------------------------------------
		// Methods : WP Hooks and Filters
		//-------------------------------------

		/**
		 * Initialize all our Admin mode goodness.
		 *
		 * This is put into the admin_init stack of WordPress via the admin_menu hook.
		 * It must come AFTER admin_menu to ensure the base plugin has been initialized.
		 *
		 * We call admin_init in the slpuml to wire this in, then relegate the rest to
		 * this class as this entire class file only loads into RAM when admin_init is
		 * called.  This prevents the base add-on pack from carrying around excess
		 * admin weight when processing front-end pages.
		 *
		 */
		function admin_init() {

			$this->debugMP('msg',__FUNCTION__.' started.');

			// Check the installed version,
			// if newer than installed version run some update stuff.
			//
			if(version_compare($this->addon->options['installed_version'], SLPUserManagedLocations::VERSION, '<')) {
				$this->debugMP('msg','',"Updating plugin from version " . $this->addon->options['installed_version'] . " to " . SLPUserManagedLocations::VERSION);

				if (class_exists('SLPUML_Activation') == false) {
					require_once('class.activation.php');
				}
				$this->activation = new SLPUML_Activation(
											array(
												'parent'    => $this,
												'slplus'    => $this->slplus,
												'addon'     => $this->parent,
											));
				$this->createshorthand_SEDataObject();
				$this->activation->update();

				$this->addon->options['installed_version'] = SLPUserManagedLocations::VERSION;
				update_option(SLPUserManagedLocations::OPTION_NAME,$this->addon->options);

			}

			// WordPress Update Checker - if this plugin is active
			// See if there is a newer version out there somewhere
			// over the rainbow.
			//
			if (is_plugin_active($this->addon->slug)) {
				$this->addon->metadata = get_plugin_data(__FILE__, false, false);
				$this->Updates = new SLPlus_Updates(
						$this->addon->metadata['Version'],
						$this->slplus->updater_url,
						$this->addon->slug
						);
			}

			// wpCSL Filters
			//
			// wpcsl_admin_slugs : skin and script the admin UI
			//
			add_filter('wpcsl_admin_slugs'              ,array($this,'filter_AddOurAdminSlug'                        ) );

			// SLP Action Hooks
			//
			// slp_location_added : update extendo data when adding a location
			// slp_location_saved : update extendo data when changing a location
			//
			add_action('slp_location_added'             ,array($this,'action_SaveExtendedDataUML'                       ), 110          );
			add_action('slp_location_saved'             ,array($this,'action_SaveExtendedDataUML'                       ), 110          );
			add_action('slp_deletelocation_starting'    ,array($this,'action_DeleteExtendedDataUML'                     ), 110          );

			// SLP Filters
			//
			// slp_edit_location_right_column : add extendo fields to location add/edit form
			// slp_manage_location_columns : show extendo fields on the locations list table
			// slp_column_data : manipulate per-location data when it is rendered in the locations list table
			//
			// NOTE: Make sure these are executed AFTER the filters of Super Extendo!!!
			//
			add_filter('slp_edit_location_right_column'         ,array($this,'filter_AddExtendedDataToEditFormUML'              ),06        );
			add_filter('slp_manage_expanded_location_columns'   ,array($this,'filter_AddExtendedDataToLocationColumnsUML'       ),15        );
			add_filter('slp_column_data'                        ,array($this,'filter_ShowExtendedDataInColumnsUML'              ),85    ,3  );

			// Pro Pack Filters
			//
			if ($this->slplus->is_AddonActive('slp-pro')) {
				add_filter('slp_csv_locationdata'       , array( $this, 'filter_CheckForPreExistingIdentifierUML'  ) );
			}
		}

		/**
		 * Create the shorthand for the Super Extendo Data object.
		 */
		function createshorthand_SEDataObject() {

			$this->debugMP('msg',__FUNCTION__.' started.');

			if (!isset($this->SEData) && $this->slplus->database->is_Extended()) {
				$this->slplus->database->createobject_DatabaseExtension();
				$this->SEData = $this->slplus->database->extension;
			}
		}

		/**
		 * Add our admin pages to the valid admin page slugs.
		 *
		 * @param string[] $slugs admin page slugs
		 * @return string[] modified list of admin page slugs
		 */
		function filter_AddOurAdminSlug($slugs) {

			$this->debugMP('msg',__FUNCTION__.' started.');

			return array_merge($slugs,
					array(
						SLPUserManagedLocations::ADMIN_PAGE_SLUG,
						SLP_ADMIN_PAGEPRE.SLPUserManagedLocations::ADMIN_PAGE_SLUG,
						)
					);
		}

		/**
		 * Look to see if incoming Identifier data is already in the extended data set.
		 *
		 * @param mixed[] $location_data
		 */
		public function filter_CheckForPreExistingIdentifierUML($location_data) {

			$this->debugMP('msg',__FUNCTION__.' started.');
			return $location_data;
		}

		/**
		 * Render the User Managed Locations tab
		 */
		function render_AdminPage() {

			$this->debugMP('msg',__FUNCTION__.' started.');

			// Setup and render settings page
			//
			$this->Settings = new wpCSL_settings__slplus(
				array(
						'prefix'            => $this->slplus->prefix,
						'css_prefix'        => $this->slplus->prefix,
						'url'               => $this->slplus->url,
						'name'              => $this->slplus->name . ' - ' . $this->addon->name,
						'plugin_url'        => $this->slplus->plugin_url,
						'render_csl_blocks' => false,
						'no_save_button'    => true,
						'form_action'       => admin_url().'admin.php?page='.SLPUserManagedLocations::ADMIN_PAGE_SLUG
					)
			 );

			//-------------------------
			// Navbar Section
			//-------------------------
			$this->Settings->add_section(
				array(
					'name'          => 'Navigation',
					'div_id'        => 'navbar_wrapper',
					'description'   => $this->slplus->AdminUI->create_Navbar(),
					'innerdiv'      => false,
					'is_topmenu'    => true,
					'auto'          => false,
					'headerbar'     => false
				)
			);


			//-------------------------
			// User Manager
			//-------------------------
			$this->createobject_UserManager();
			$this->usermanager->prepare_items();
			$sectName = __('User Manager','csa-slp-uml');
			$this->Settings->add_section(
				array(
					'name'        => $sectName,
					'label'       => __('List of registered users','csa-slp-uml'),
					'show_label'  => true,
					'description' => $this->usermanager->display()
					)
				);


			//------------------------------------------
			// RENDER
			//------------------------------------------
			$this->Settings->render_settings_page();
		}

		/**
		 * Add the Store Locator panel to the admin sidebar for non-admin editors.
		 *
		 */
		function slp_user_managed_locations_admin_menu() {

			$this->debugMP('msg',__FUNCTION__.' started.');
			return;

		}

		//-------------------------------------
		// Methods : SLP Hooks
		//-------------------------------------

		/**
		 * Delete the extended data.
		 */
		function action_DeleteExtendedDataUML() {
			$this->debugMP('msg',__FUNCTION__);
			// This functionality is already handled by SLP Super Extendo hooks
		}

		/**
		 * Save the extended data.
		 */
		function action_SaveExtendedDataUML() {
			$this->debugMP('msg',__FUNCTION__);

			// Check our extended columns and see if there is a matching property in exdata in currentLocation
			$newValues = array();

			// Store the current user login in the store_user field if user is store editor and noAdmin
			//
			if ($this->addon->slp_uml_is_user(true)) {
				$current_user = wp_get_current_user();
				$newValues[SLPUserManagedLocations::SLP_UML_USER_SLUG] = $current_user->user_login;
			} else {
				$this->debugMP('msg',__FUNCTION__ . ' slp_uml_is_user() returned false so store_user is not overwritten.');
			}

			// New values?  Write them to disk...
			if (count($newValues) > 0){
				$this->debugMP('pr',__FUNCTION__,$newValues);
				$this->createshorthand_SEDataObject();
				if ($this->slplus->database->is_Extended()) {
					$this->slplus->database->extension->update_data($this->slplus->currentLocation->id, $newValues);
				}
			} else {
				$this->debugMP('msg',__FUNCTION__ . ' No extended data overwritten.');
			}
		}

		/**
		 * Hide the Store User for a User Managed Locations
		 * 
		 * @param $theform
		 */
		function filter_AddExtendedDataToEditFormUML($theform) {
			$this->debugMP('msg',__FUNCTION__.' started for current Location: ' . $this->slplus->currentLocation->id);

			// Make the User Managed Locations columns hidden if store editor
			if ($this->addon->slp_uml_is_user(true)) {

				// Find and hide the label
				$patternText = "/label for='".SLPUserManagedLocations::SLP_UML_USER_SLUG."/";
				$replaceText = "label style='visibility:hidden!important' for=".SLPUserManagedLocations::SLP_UML_USER_SLUG;
				$new_form_tmp = preg_replace($patternText, $replaceText, $theform);
				$this->debugMP('msg','',' Made SED columns hidden, searched for:' . $patternText);

				// Find and hide the input field and set the default value to current user login value
				$curStoreUser = wp_get_current_user();
				$patternText = "/input type='text' id='edit-".SLPUserManagedLocations::SLP_UML_USER_SLUG."'([^>]*) value='[^>]*' /";
				$replaceText = "input type='hidden' id='edit-".SLPUserManagedLocations::SLP_UML_USER_SLUG."'$1 value='".$curStoreUser->user_login."' ";
				$new_form = preg_replace($patternText, $replaceText, $new_form_tmp);
				$this->debugMP('msg','',' Made SED columns hidden, searched for:' . $patternText);
				$this->debugMP('msg','',' Made SED columns hidden, to be replaced by:' . $replaceText);

			} else {
				// No changes
				$new_form = $theform;
			}

			return $new_form;
		}

		/**
		 * Removes the User Managed Locations columns for a User Managed Locations
		 * @param $current_cols array The current columns
		 */
		function filter_AddExtendedDataToLocationColumnsUML($current_cols) {
			$this->debugMP('pr',__FUNCTION__.' started for current_cols:',$current_cols);

			// Remove User Managed Locations columns if allowed user
			//if (!$this->slpuml->slp_uml_is_admin()) {
			if ($this->addon->slp_uml_is_user(true)) {
				$new_cols = $this->remove_UserManagedLocations_Fields($current_cols);
				$this->debugMP('pr',__FUNCTION__.' removed SED columns new_cols:',$new_cols);
			} else {
				// No changes
				$new_cols = $current_cols;
			}
			return $new_cols;
		}

		/**
		 * Allows editing of the extendo data for the location
		 * 
		 * @param $thedata  - option value field
		 * @param $thefield - The name of the field
		 * @param $thelabel - The column label
		 */
		function filter_ShowExtendedDataInColumnsUML($thedata, $thefield, $thelabel) {
	//		$this->debugMP('pr',__FUNCTION__.' started for theData:',$thedata);

			// This functionality is already handled by SLP Super Extendo hooks

			return $thedata;
		}

		//-------------------------------------
		// Methods : Object Management
		//-------------------------------------

		/**
		 * Create the data interface object.
		 */
		function remove_UserManagedLocations_Fields($sedColumns = array()) {
			$this->debugMP('msg',__FUNCTION__.' started.');

			// Remove the User Managed Locations fields from the set of columns
			unset($sedColumns[SLPUserManagedLocations::SLP_UML_USER_SLUG]);

			return $sedColumns;
		}

		/**
		 * Create the data interface object.
		 */
		function createobject_UserManager() {
			$this->debugMP('msg',__FUNCTION__.' started.');
			if (class_exists('SLPUML_AdminUI_UserManager') == false) {
				require_once('class.adminui.user_manager.php');
			}
			if (!isset($this->usermanager)) {
				$this->usermanager = new SLPUML_AdminUI_UserManager(array('slpuml' => $this->addon));
	//            $this->debugMP('msg',__FUNCTION__ . ' created:',$this->usermanager);
			}
		}

		/**
		 * Simplify the plugin debugMP interface.
		 *
		 * Typical start of function call: $this->debugMP('msg',__FUNCTION__);
		 *
		 * @param string $type
		 * @param string $hdr
		 * @param string $msg
		 */
		function debugMP($type,$hdr,$msg='') {
			if (($type === 'msg') && ($msg!=='')) {
				$msg = esc_html($msg);
			}
			if (($hdr!=='')) {
				$hdr = 'AdminUI: ' . $hdr;
			}
			$this->addon->debugMP($type,$hdr,$msg,NULL,NULL,true);
		}
	}
}
